//*****************************************************************************
/// @file
/// @brief
///   Arduino SmartThings Shield Washer/Dryer Altert
/// @note
///              ______________
///             |              |
///             |         SW[] |
///             |[]RST         |
///             |         AREF |--
///             |          GND |--
///             |           13 |--X LED
///             |           12 |--
///             |           11 |--
///           --| 3.3V      10 |--
///           --| 5V         9 |--
///           --| GND        8 |--
///           --| GND          |
///           --| Vin        7 |--
///             |            6 |--
///           --| A0         5 |--
///           --| A1    ( )  4 |--
///           --| A2         3 |--X THING_RX
///           --| A3  ____   2 |--X THING_TX
///           --| A4 |    |  1 |--
///           --| A5 |    |  0 |--
///             |____|    |____|
///                  |____|
///
//*****************************************************************************
#include <SoftwareSerial.h>   //TODO need to set due to some weird wire language linker, should we absorb this whole library into smartthings
#include <SmartThings.h>

//*****************************************************************************
// Pin Definitions    | | | | | | | | | | | | | | | | | | | | | | | | | | | | |
//                    V V V V V V V V V V V V V V V V V V V V V V V V V V V V V
//*****************************************************************************
#define PIN_LED                13
#define PIN_THING_RX            3
#define PIN_THING_TX            2

#define PIN_WASHER_LED          4
#define PIN_WASHER_PHOTOSENSOR  0

#define PIN_DRYER_LED           5
#define PIN_DRYER_PHOTOSENSOR   1

//*****************************************************************************
// Global Variables   | | | | | | | | | | | | | | | | | | | | | | | | | | | | |
//                    V V V V V V V V V V V V V V V V V V V V V V V V V V V V V
//*****************************************************************************
SmartThingsCallout_t messageCallout;    // call out function forward decalaration
SmartThings smartthing(PIN_THING_RX, PIN_THING_TX, messageCallout);  // constructor

bool isDebugEnabled;       // enable or disable debug in this example
int  stateLED;             // state to track last set value of LED

int  dryerState  = 0;      // True if the LED is on; false if off
int  washerState = 0;      // True if the LED is on; false if off

//*****************************************************************************
// API Functions    | | | | | | | | | | | | | | | | | | | | | | | | | | | | | |
//                  V V V V V V V V V V V V V V V V V V V V V V V V V V V V V V
//*****************************************************************************
void setup()
{
  // setup default state of global variables
  isDebugEnabled = true;
  stateLED = 0;                        // matches state of hardware pin set below
  
  // setup hardware pins 
  pinMode(PIN_LED, OUTPUT);            // define PIN_LED as an output
  digitalWrite(PIN_LED, LOW);          // set value to LOW (off) to match stateLED=0

  pinMode(PIN_WASHER_LED, OUTPUT);     // define PIN_WASHER_LED as an output
  digitalWrite(PIN_WASHER_LED, LOW);   // set value to LOW (off) to match drayerState=0

  pinMode(PIN_DRYER_LED, OUTPUT);      // define PIN_DRYER_LED as an output
  digitalWrite(PIN_DRYER_LED, LOW);    // set value to LOW (off) to match washerState=0

  if (isDebugEnabled)
  { // setup debug serial port
    Serial.begin(9600);                // setup serial with a baud rate of 9600
    Serial.println("setup...");        // print out 'setup..' on start
  }
}

//*****************************************************************************
// Test the sensor and send a message to SmartThings.  The message is in the format of
//  "name:state", where "name" is usually "washer" or "dryer" and "state" is either
//  "pending" or "done".
int testSensor( int sensorPin, int ledPin, int oldState, const char *name )
{
  int     value    = analogRead( sensorPin );
  int     newState = (value > 512) ? HIGH : LOW;
  String  toSend;
  

  if( newState != oldState ) {
    if( isDebugEnabled ) {
      Serial.print( name );
      Serial.print( " photosensor state changed to " );
      Serial.print( newState == HIGH ? "high" : "low" );
      Serial.println( "; updating the LED" );
      digitalWrite( ledPin, newState );
    }

    // Compose the string in the form of "appliance:applianceEvent".  The part after the colon
    //  Matches the command/state in the SmartThings Device Type, mostly so we can change the
    //  colorof the tile.
    toSend  = name;
    toSend += ":";
    toSend += (newState == HIGH) ? "done" : "pending";

    // Send the message
    
    if( isDebugEnabled ) {
      Serial.print( "Sending " );
      Serial.println( toSend );
    }
    smartthing.send( toSend );
  }

  return newState;
}

//*****************************************************************************
// Main loop.  We just test the sensor and send the event.
void loop()
{
  // run smartthing logic
  smartthing.run();

  // Check the value of the photosensor
  dryerState  = testSensor( PIN_DRYER_PHOTOSENSOR,  PIN_DRYER_LED,  dryerState,  "dryer"  );
  washerState = testSensor( PIN_WASHER_PHOTOSENSOR, PIN_WASHER_LED, washerState, "washer" );
}

//*****************************************************************************
// Called when SmartThings wants to tell us something.  We don't have any use for this,
//  so we just print it right now.
void messageCallout(String message)
{
  // if debug is enabled print out the received message
  if (isDebugEnabled)
  {
    Serial.print("Received message: '");
    Serial.print(message);
    Serial.println("' ");
  }
}

